@@ -204,11 +204,17 @@ class Operand:
 
     :ivar is_input: indicates if the instruction reads the variable specified by this operand.
     :ivar is_output: indicates if the instruction writes the variable specified by this operand.
+    :ivar extended_size: for immediate operands the size of the value in bytes after size-extension.
+
+        The extended size affects which operand values can be encoded. E.g. a signed imm8 operand 
+        would normally values in the [-128, 127] range. But if it is extended to 4 bytes, it can also
+        encode values in [2**32 - 128, 2**32 - 1] range.
     """
     def __init__(self, type):
         self.type = type
         self.is_input = False
         self.is_output = False
+        self.extended_size = None
 
     def __str__(self):
         """Return string representation of the operand type and its read/write attributes"""
@@ -237,6 +243,11 @@ class Operand:
         """Indicates whether this operand specifies a memory location"""
         return self.type in {"m", "m8", "m16", "m32", "m64", "m80", "m128", "m256", "m512", "vm32x", "vm32y", "vm64x", "vm64y"}
 
+    @property
+    def is_immediate(self):
+        """Indicates whether this operand is an immediate constant"""
+        return self.type in {"imm4", "imm8", "imm16", "imm32"}
+
 
 class ISAExtension:
     _score_map = {
@@ -510,13 +521,10 @@ class ModRM:
         If mode is a reference to an instruction operand, the operand has memory type and its addressing mode must be \
         coded instruction the Mod R/M mode field.
 
-    :ivar rm: a register or memory operand. Possible values are None or a reference to an instruction operand.
+    :ivar rm: a register or memory operand. Must be a reference to an instruction operand.
 
-        If rm is None, the field is ignored. Only one form of EXTRQ instruction from SSE4A uses this value.
-
-        If rm is a reference to an instruction operand, and the operand is of register type, rm specifies bits 0-2 of \
-        the register number. If the operand is of memory type, rm specifies bits 0-2 of the base register number \
-        unless a SIB byte is used.
+        If rm is a reference to a operand, rm specifies bits 0-2 of the register number. If the operand is of memory \
+        type, rm specifies bits 0-2 of the base register number unless a SIB byte is used.
 
     :ivar reg: a register or an opcode extension. Possible values are an int value, or a reference to an instruction \
         operand.
@@ -628,6 +636,8 @@ def read_instruction_set(filename=os.path.join(os.path.dirname(os.path.abspath(_
                 operand = Operand(xml_operand.attrib["type"])
                 operand.is_input = _bool(xml_operand.attrib.get("input", "false"))
                 operand.is_output = _bool(xml_operand.attrib.get("output", "false"))
+                if "extended-size" in xml_operand.attrib:
+                    operand.extended_size = int(xml_operand.attrib.get("extended-size"))
                 instruction_form.operands.append(operand)
             for xml_implicit_operand in xml_instruction_form.findall("ImplicitOperands"):
                 if _bool(xml_implicit_operand.attrib["input"]):
@@ -711,14 +721,8 @@ def read_instruction_set(filename=os.path.join(os.path.dirname(os.path.abspath(_
                         else:
                             assert "reg-operand-number" in xml_component.attrib
                             modrm.reg = instruction_form.operands[int(xml_component.attrib["reg-operand-number"])]
-                        if "rm" in xml_component.attrib:
-                            assert "rm-operand-number" not in xml_component.attrib
-                            assert xml_component.attrib["rm"] == "ignored"
-                            modrm.rm = None
-                        else:
-                            assert "rm-operand-number" in xml_component.attrib
-                            modrm.rm = instruction_form.operands[int(xml_component.attrib["rm-operand-number"])]
-                            encoding.components.append(modrm)
+                        modrm.rm = instruction_form.operands[int(xml_component.attrib["rm-operand-number"])]
+                        encoding.components.append(modrm)
                     elif xml_component.tag == "Immediate":
                         assert "size" in xml_component.attrib
                         immediate = Immediate()
