@@ -63,6 +63,13 @@ class InstructionForm:
 
         Instruction forms with cancelling inputs have only two input operands, which have the same register type.
 
+    :ivar nacl_version: indicates the earliest Pepper API version where validator supports this instruction.
+
+        Possible values are integers >= 33 or None. Pepper 33 is the earliest version for which information on
+        supported instructions is available; if instruction forms supported before Pepper 33 would have
+        nacl_version == 33. None means instruction is either not yet supported by Native Client validator, or
+        is forbidden in Native Client SFI model.
+
     :ivar nacl_zero_extends_outputs: indicates that Native Client validator recognizes that the instruction zeroes 
         the upper 32 bits of the output registers.
 
@@ -84,6 +91,7 @@ class InstructionForm:
         self.mmx_mode = None
         self.xmm_mode = None
         self.cancelling_inputs = None
+        self.nacl_version = None
         self.nacl_zero_extends_outputs = None
         self.operands = []
         self.implicit_inputs = set()
@@ -229,11 +237,17 @@ class Operand:
 
     :ivar is_input: indicates if the instruction reads the variable specified by this operand.
     :ivar is_output: indicates if the instruction writes the variable specified by this operand.
+    :ivar extended_size: for immediate operands the size of the value in bytes after size-extension.
+
+        The extended size affects which operand values can be encoded. E.g. a signed imm8 operand 
+        would normally values in the [-128, 127] range. But if it is extended to 4 bytes, it can also
+        encode values in [2**32 - 128, 2**32 - 1] range.
     """
     def __init__(self, type):
         self.type = type
         self.is_input = False
         self.is_output = False
+        self.extended_size = None
 
     def __str__(self):
         """Return string representation of the operand type and its read/write attributes"""
@@ -262,6 +276,11 @@ class Operand:
         """Indicates whether this operand specifies a memory location"""
         return self.type in {"m", "m8", "m16", "m32", "m64", "m80", "m128", "m256", "m512", "vm32x", "vm32y", "vm64x", "vm64y"}
 
+    @property
+    def is_immediate(self):
+        """Indicates whether this operand is an immediate constant"""
+        return self.type in {"imm4", "imm8", "imm16", "imm32", "imm64"}
+
 
 class ISAExtension:
     _score_map = {
@@ -620,13 +639,10 @@ class ModRM:
         If mode is a reference to an instruction operand, the operand has memory type and its addressing mode must be \
         coded instruction the Mod R/M mode field.
 
-    :ivar rm: a register or memory operand. Possible values are None or a reference to an instruction operand.
+    :ivar rm: a register or memory operand. Must be a reference to an instruction operand.
 
-        If rm is None, the field is ignored. Only one form of EXTRQ instruction from SSE4A uses this value.
-
-        If rm is a reference to an instruction operand, and the operand is of register type, rm specifies bits 0-2 of \
-        the register number. If the operand is of memory type, rm specifies bits 0-2 of the base register number \
-        unless a SIB byte is used.
+        If rm is a reference to a operand, rm specifies bits 0-2 of the register number. If the operand is of memory \
+        type, rm specifies bits 0-2 of the base register number unless a SIB byte is used.
 
     :ivar reg: a register or an opcode extension. Possible values are an int value, or a reference to an instruction \
         operand.
@@ -711,7 +727,6 @@ def _bool(xml_boolean):
         "The boolean string must be \"true\" or \"false\""
     return {"true": True, "false": False}[xml_boolean]
 
-
 def read_instruction_set(filename=os.path.join(os.path.dirname(os.path.abspath(__file__)), "x86_64.xml")):
     """Reads instruction set data from an XML file and returns a list of :class:`Instruction` objects
 
@@ -734,12 +749,16 @@ def read_instruction_set(filename=os.path.join(os.path.dirname(os.path.abspath(_
             instruction_form.mmx_mode = xml_instruction_form.attrib.get("mmx-mode")
             instruction_form.xmm_mode = xml_instruction_form.attrib.get("xmm-mode")
             instruction_form.cancelling_inputs = _bool(xml_instruction_form.attrib.get("cancelling-inputs", "false"))
+            if "nacl-version" in xml_instruction_form.attrib:
+                instruction_form.nacl_version = int(xml_instruction_form.attrib["nacl-version"])
             if "nacl-zero-extends-outputs" in xml_instruction_form.attrib:
                 instruction_form.nacl_zero_extends_outputs = _bool(xml_instruction_form.attrib["nacl-zero-extends-outputs"])
             for xml_operand in xml_instruction_form.findall("Operand"):
                 operand = Operand(xml_operand.attrib["type"])
                 operand.is_input = _bool(xml_operand.attrib.get("input", "false"))
                 operand.is_output = _bool(xml_operand.attrib.get("output", "false"))
+                if "extended-size" in xml_operand.attrib:
+                    operand.extended_size = int(xml_operand.attrib.get("extended-size"))
                 instruction_form.operands.append(operand)
             for xml_implicit_operand in xml_instruction_form.findall("ImplicitOperands"):
                 if _bool(xml_implicit_operand.attrib["input"]):
@@ -908,14 +927,8 @@ def read_instruction_set(filename=os.path.join(os.path.dirname(os.path.abspath(_
                         else:
                             assert "reg-operand-number" in xml_component.attrib
                             modrm.reg = instruction_form.operands[int(xml_component.attrib["reg-operand-number"])]
-                        if "rm" in xml_component.attrib:
-                            assert "rm-operand-number" not in xml_component.attrib
-                            assert xml_component.attrib["rm"] == "ignored"
-                            modrm.rm = None
-                        else:
-                            assert "rm-operand-number" in xml_component.attrib
-                            modrm.rm = instruction_form.operands[int(xml_component.attrib["rm-operand-number"])]
-                            encoding.components.append(modrm)
+                        modrm.rm = instruction_form.operands[int(xml_component.attrib["rm-operand-number"])]
+                        encoding.components.append(modrm)
                     elif xml_component.tag == "Immediate":
                         assert "size" in xml_component.attrib
                         immediate = Immediate()
